"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeUnreachable = void 0;
function addReachable(definition, definitions, reachable, empties) {
    if (typeof definition === "boolean" || Object.keys(definition).length === 0) {
        return;
    }
    if (definition === null || definition === void 0 ? void 0 : definition.$ref) {
        const $ref = decodeURIComponent(definition.$ref);
        if (reachable.has($ref)) {
            return;
        }
        if (empties.has($ref) || isEmpty($ref, definitions)) {
            delete definition.$ref;
            empties.add($ref);
        }
        else {
            reachable.add($ref);
            addReachable(definitions[$ref.slice(14)], definitions, reachable, empties);
        }
    }
    else if (definition.anyOf) {
        for (const def of definition.anyOf) {
            addReachable(def, definitions, reachable, empties);
        }
    }
    else if (definition.allOf) {
        for (const def of definition.allOf) {
            addReachable(def, definitions, reachable, empties);
        }
    }
    else if (definition.oneOf) {
        for (const def of definition.oneOf) {
            addReachable(def, definitions, reachable, empties);
        }
    }
    else if (definition.not) {
        addReachable(definition.not, definitions, reachable, empties);
    }
    else if (definition.type === "object") {
        for (const prop in definition.properties || {}) {
            const propDefinition = definition.properties[prop];
            addReachable(propDefinition, definitions, reachable, empties);
        }
        const additionalProperties = definition.additionalProperties;
        if (additionalProperties) {
            addReachable(additionalProperties, definitions, reachable, empties);
        }
    }
    else if (definition.type === "array") {
        const items = definition.items;
        if (Array.isArray(items)) {
            for (const item of items) {
                addReachable(item, definitions, reachable, empties);
            }
        }
        else if (items) {
            addReachable(items, definitions, reachable, empties);
        }
    }
}
function isEmpty(ref, definitions) {
    const target = definitions[decodeURIComponent(ref).slice(14)];
    if (target === null || target === undefined) {
        return true;
    }
    else {
        const targetPropCount = Object.keys({ ...target }).length;
        if (targetPropCount === 0) {
            return true;
        }
        else if (target === null || target === void 0 ? void 0 : target.$ref) {
            if (isEmpty(target.$ref, definitions) && targetPropCount === 1) {
                return true;
            }
        }
    }
    return false;
}
function removeUnreachable(rootTypeDefinition, definitions) {
    if (!rootTypeDefinition) {
        return definitions;
    }
    const reachable = new Set();
    const empties = new Set();
    addReachable(rootTypeDefinition, definitions, reachable, empties);
    const out = {};
    for (const def of reachable) {
        const key = decodeURIComponent(def).slice(14);
        out[key] = definitions[key];
    }
    return out;
}
exports.removeUnreachable = removeUnreachable;
//# sourceMappingURL=removeUnreachable.js.map